package djmil.cordacheckers.gamestate;

import java.util.List;
import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.checkers.Stone;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameProposalState;
import djmil.cordacheckers.states.GameResultState;
import net.corda.v5.base.types.MemberX500Name;

// GameBoard from the player's point of view
public class View {
    public static enum Status {
        GAME_PROPOSAL_WAIT_FOR_OPPONENT,
        GAME_PROPOSAL_WAIT_FOR_YOU,
        GAME_PROPOSAL_REJECTED,
        GAME_PROPOSAL_CANCELED,
        
        GAME_BOARD_WAIT_FOR_OPPONENT,
        GAME_BOARD_WAIT_FOR_YOU,

        DRAW_REQUEST_WAIT_FOR_OPPONENT,
        DRAW_REQUEST_WAIT_FOR_YOU,

        GAME_RESULT_YOU_WON,
        GAME_RESULT_YOU_LOOSE,
        GAME_RESULT_DRAW;
    }
    public final Status status;

    public final Stone.Color myColor;
    public final String opponentName;

    public final Map<Integer, Stone> board;
    public final Integer moveNumber;
    public final List<Integer> previousMove;

    public final String message;
    public final UUID uuid;

    // Serialisation service requires a default constructor
    public View() {
        this.status = null;
        this.opponentName = null;
        this.myColor = null;
        this.board = null;
        this.moveNumber = null;
        this.previousMove = null;
        this.message = null;
        this.uuid = null;
    }

    public View(View.Status status, MemberX500Name myName, GameProposalState gameProposal) {
        this.status = status;
        this.opponentName = gameProposal.getOpponentName(myName).getCommonName();
        this.myColor = gameProposal.getPlayerColor(myName);
        this.board = gameProposal.getBoard();
        this.previousMove = null;
        this.moveNumber = 0;
        this.message = gameProposal.getMessage();
        this.uuid = gameProposal.getGameUuid();
    }

    public View(View.Status status, MemberX500Name myName, GameBoardState gameBoard, List<Integer> previousMove) {
        this.status = status;
        this.opponentName = gameBoard.getOpponentName(myName).getCommonName();
        this.myColor = gameBoard.getPlayerColor(myName);
        this.board = gameBoard.getBoard();
        this.previousMove = previousMove;
        this.moveNumber = gameBoard.getMoveNumber();
        this.message = gameBoard.getMessage();
        this.uuid = gameBoard.getGameUuid();
    }

    public View(View.Status status, MemberX500Name myName, GameResultState gameResult) {
        this.status = status;
        this.opponentName = gameResult.getOpponentName(myName).getCommonName();
        this.myColor = gameResult.getPlayerColor(myName);
        this.board = null;
        this.previousMove = null;
        this.moveNumber = gameResult.getTotalMoves();
        this.message = gameResult.getMessage();
        this.uuid = gameResult.getGameUuid();
    }

}
